import os
from .constants import (
    MODEL,
    DATA,
    OPTIMIZATION,
    ENVIRONMENT,
)


def list_model_presets():
    """
    List all available model types.
    Image/text backbones can be customized for one model type.

    Returns
    -------
    A list of model types.
    """
    cur_path = os.path.dirname(os.path.abspath(__file__))
    model_config_dir = os.path.join(cur_path, "configs", "model")
    model_config_files = [f for f in os.listdir(model_config_dir) if f.endswith((".yaml", ".yml"))]
    model_presets = [f.split(".")[0] for f in model_config_files]
    return model_presets


def preset_to_config(model_type: str):
    """
    Get the default config of one predictor in AutoMM.
    Currently, we only use model type to differentiate different predictors.
    In future, we can simultaneously consider model type, data type, optimization type,
    and environment type to construct more diverse presets.

    Parameters
    ----------
    model_type
        Type of model supported by AutoMM.

    Returns
    -------
    A predictor's preset config strings of MODEL, DATA, OPTIMIZATION, and ENVIRONMENT.
    """
    model_type = model_type.lower()
    config = {
        MODEL: "",
        DATA: "default",
        OPTIMIZATION: "adamw",
        ENVIRONMENT: "default",
    }

    if model_type == "text":
        config[MODEL] = "hf_text"
    elif model_type == "image":
        config[MODEL] = "timm_image"
    else:
        config[MODEL] = model_type

        cur_path = os.path.dirname(os.path.abspath(__file__))
        model_config_dir = os.path.join(cur_path, "configs", "model")
        model_config_path = os.path.join(model_config_dir, f"{model_type}.yaml")
        if not os.path.isfile(model_config_path):
            raise ValueError(
                f"Model type '{model_type}' is not supported yet. "
                f"Consider one of these: {list_model_presets()}"
            )

    return config



from ..automm.presets import preset_to_config


def list_text_presets(verbose=False):
    """
    Returns
    -------
    If verbose==True, return all the preset strings and their corresponding config customizations.
    If verbose==False, return a list of simple presets strings.
    """
    simple_presets = {
        "default": {
            "model.hf_text.checkpoint_name": "google/electra-base-discriminator",
        },
        "medium_quality_faster_train": {
            "model.hf_text.checkpoint_name": "google/electra-small-discriminator",
            "optimization.learning_rate": 4e-4,
        },
        "high_quality": {
            "model.hf_text.checkpoint_name": "google/electra-base-discriminator",
        },
        "best_quality": {
            "model.hf_text.checkpoint_name": "microsoft/deberta-v3-base",
            "env.per_gpu_batch_size": 2,
        },
        "multilingual": {
            "model.hf_text.checkpoint_name": "microsoft/mdeberta-v3-base",
            "env.precision": "bf16",
            "env.per_gpu_batch_size": 2,
        },
    }

    if verbose:
        return simple_presets
    else:
        return list(simple_presets.keys())


def text_preset_to_config(preset: str):
    """
    Convert a preset string to AutoMM's config.

    Parameters
    ----------
    preset
        A simple preset string, which should be covered by list_presets().

    Returns
    -------
    config
        A config dictionary of AutoMM.
    overrides
        A dictionary with customized backbones.
    """
    model_type = "fusion_mlp_text_tabular"
    config = preset_to_config(model_type)
    preset = preset.lower()
    available_presets = list_text_presets(verbose=True)

    if preset in available_presets:
        overrides = available_presets[preset]
    else:
        raise ValueError(
            f"Provided preset '{preset}' is not supported. "
            f"Consider one of these: {list_text_presets()}"
        )

    return config, overrides
